@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                            )--------------------
-# Copyright (c) 2014-2023 Beartype authors.
+# Copyright (c) 2014-2025 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
@@ -19,16 +19,17 @@ This private submodule is *not* intended for importation by downstream callers.
 # submodule to improve maintainability and readability here.
 
 # ....................{ IMPORTS                            }....................
-from beartype.typing import TYPE_CHECKING
-from beartype._conf.confcls import (
-    BEARTYPE_CONF_DEFAULT,
-    BeartypeConf,
+from beartype.typing import (
+    TYPE_CHECKING,
+    Callable,
 )
+from beartype._conf.confmain import BeartypeConf
+from beartype._conf.confcommon import BEARTYPE_CONF_DEFAULT
 from beartype._data.hint.datahinttyping import (
-    BeartypeConfedDecorator,
     BeartypeReturn,
     BeartypeableT,
 )
+from beartype._util.py.utilpyinterpreter import is_python_optimized
 
 # Intentionally import the standard mypy-friendly @typing.overload decorator
 # rather than a possibly mypy-unfriendly @beartype.typing.overload decorator --
@@ -41,11 +42,19 @@ from typing import overload
 # @beartype decorator declared below is permissively annotated as returning a
 # union of multiple types desynchronized from the types of the passed arguments
 # and thus fails to accurately convey the actual public API of that decorator.
-# See also: https://www.python.org/dev/peps/pep-0484/#function-method-overloading
+# See also:
+#     https://www.python.org/dev/peps/pep-0484/#function-method-overloading
+#
+# Note that the "Callable[[BeartypeableT], BeartypeableT]" type hint should
+# ideally instead be a reference to our "BeartypeConfedDecorator" type hint.
+# Indeed, it used to be. Unfortunately, a significant regression in mypy
+# required us to inline that type hint away. See also this issue:
+#     https://github.com/beartype/beartype/issues/332
 @overload  # type: ignore[misc,no-overload-impl]
 def beartype(obj: BeartypeableT) -> BeartypeableT: ...
 @overload
-def beartype(*, conf: BeartypeConf) -> BeartypeConfedDecorator: ...
+def beartype(*, conf: BeartypeConf) -> Callable[
+    [BeartypeableT], BeartypeableT]: ...
 
 # ....................{ DECORATORS                         }....................
 #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
@@ -67,8 +76,12 @@ if (
     # runtime context under which @beartype is only ever run. Nonetheless, this
     # test is only performed once per process and is thus effectively free.
     TYPE_CHECKING or
-    # Optimized (e.g., option "-O" was passed to this interpreter) *OR*...
-    not __debug__
+    # Optimized at process invocation time (e.g., at least one "-O" command-line
+    # option was set when this interpreter forked) *OR*...
+    not __debug__ or
+    # Optimized *AFTER* process invocation time (e.g., in an interactive REPL by
+    # the external user). Yes, our awesome userbase actually requested this.
+    is_python_optimized()
 ):
 # Then unconditionally disable @beartype-based type-checking across the entire
 # codebase by reducing the @beartype decorator to the identity decorator.
@@ -146,15 +159,16 @@ beartype.__doc__ = (
     ----------
     obj : Optional[BeartypeableT]
         **Beartypeable** (i.e., pure-Python callable or class) to be decorated.
-        Defaults to ``None``, in which case this decorator is in configuration
-        rather than decoration mode. In configuration mode, this decorator
-        creates and returns an efficiently cached private decorator that
-        generically applies the passed beartype configuration to any
+        Defaults to :data:`None`, in which case this decorator is in
+        configuration rather than decoration mode. In configuration mode, this
+        decorator creates and returns an efficiently cached private decorator
+        that generically applies the passed beartype configuration to any
         beartypeable object passed to that decorator. Look... It just works.
     conf : BeartypeConf, optional
         **Beartype configuration** (i.e., self-caching dataclass encapsulating
         all settings configuring type-checking for the passed object). Defaults
-        to ``BeartypeConf()``, the default ``O(1)`` constant-time configuration.
+        to ``BeartypeConf()``, the default :math:`O(1)` constant-time
+        configuration.
 
     Returns
     ----------
@@ -162,15 +176,15 @@ beartype.__doc__ = (
         Either:
 
         * If in decoration mode (i.e., ``obj`` is *not* ``None` while ``conf``
-          is ``None``) *and*:
+          is :data:`None`) *and*:
 
           * If ``obj`` is a callable, a new callable wrapping that callable
             with dynamically generated type-checking.
           * If ``obj`` is a class, this existing class embellished with
             dynamically generated type-checking.
 
-        * If in configuration mode (i.e., ``obj`` is ``None` while ``conf`` is
-          *not* ``None``), a new beartype decorator enabling this
+        * If in configuration mode (i.e., ``obj`` is :data:`None` while ``conf``
+          is *not* :data:`None`), a new beartype decorator enabling this
           configuration.
 
     Raises
