@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                            )--------------------
-# Copyright (c) 2014-2024 Beartype authors.
+# Copyright (c) 2014-2025 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
@@ -17,12 +17,12 @@ classes encapsulating sample type hints instantiated by the
 from beartype.typing import (
     Iterable,
     Optional,
+    Tuple,
     Type,
+    TypeVar,
 )
-from beartype._conf.confcls import (
-    BEARTYPE_CONF_DEFAULT,
-    BeartypeConf,
-)
+from beartype._conf.confmain import BeartypeConf
+from beartype._conf.confcommon import BEARTYPE_CONF_DEFAULT
 from collections.abc import Iterable as IterableABC
 
 # ....................{ CLASSES ~ hint : [un]satisfied     }....................
@@ -183,13 +183,6 @@ class HintNonpepMetadata(object):
     conf : BeartypeConf
         **Beartype configuration** (i.e., self-caching dataclass encapsulating
         all settings configuring type-checking for this type hint).
-    warning_type : Optional[Type[Warning]]
-        Either:
-
-        * If the :func:`beartype.beartype` decorator unconditionally emits a
-          non-fatal warning when this type hint annotates *any* parameter or
-          return of *any* callable, the type of that warning.
-        * Else, :data:`None`.
     is_ignorable : bool
         :data:`True` only if this hint is safely ignorable by the
         :func:`beartype.beartype` decorator. Defaults to :data:`False`.
@@ -208,6 +201,13 @@ class HintNonpepMetadata(object):
         arbitrary object either satisfying or violating this hint when passed as
         a parameter *or* returned as a value annotated by this hint. Defaults to
         the empty tuple.
+    warning_type : Optional[Type[Warning]]
+        Either:
+
+        * If the :func:`beartype.beartype` decorator unconditionally emits a
+          non-fatal warning when this type hint annotates *any* parameter or
+          return of *any* callable, the type of that warning.
+        * Else, :data:`None`.
     '''
 
     # ..................{ INITIALIZERS                       }..................
@@ -220,18 +220,16 @@ class HintNonpepMetadata(object):
 
         # Optional keyword-only parameters.
         conf: BeartypeConf = BEARTYPE_CONF_DEFAULT,
-        warning_type: Optional[Type[Warning]] = None,
         is_ignorable: bool = False,
         is_needs_cls_stack: bool = False,
         is_supported: bool = True,
         piths_meta: Iterable[HintPithSatisfiedMetadata] = (),
+        warning_type: Optional[Type[Warning]] = None,
     ) -> None:
 
         # Validate passed non-variadic parameters.
         assert isinstance(conf, BeartypeConf), (
             f'{repr(conf)} not configuration.')
-        assert isinstance(warning_type, _NoneTypeOrType), (
-            f'{repr(warning_type)} neither class nor "None".')
         assert isinstance(is_ignorable, bool), (
             f'{repr(is_ignorable)} not bool.')
         assert isinstance(is_needs_cls_stack, bool), (
@@ -247,27 +245,29 @@ class HintNonpepMetadata(object):
             f'{repr(piths_meta)} not iterable of '
             f'"HintPithSatisfiedMetadata" and '
             f'"HintPithUnsatisfiedMetadata" instances.')
+        assert isinstance(warning_type, _NoneTypeOrType), (
+            f'{repr(warning_type)} neither class nor "None".')
 
         # Classify all passed parameters.
         self.hint = hint
         self.conf = conf
-        self.warning_type = warning_type
         self.is_ignorable = is_ignorable
         self.is_needs_cls_stack = is_needs_cls_stack
         self.is_supported = is_supported
         self.piths_meta = piths_meta
+        self.warning_type = warning_type
 
-    # ..................{ STRINGIFIERS                       }..................
+    # ..................{ DUNDERS                            }..................
     def __repr__(self) -> str:
         return '\n'.join((
             f'{self.__class__.__name__}(',
             f'    hint={repr(self.hint)},',
             f'    conf={repr(self.conf)},',
-            f'    warning_type={repr(self.warning_type)},',
             f'    is_ignorable={repr(self.is_ignorable)},',
             f'    is_needs_cls_stack={repr(self.is_needs_cls_stack)},',
             f'    is_supported={repr(self.is_supported)},',
             f'    piths_meta={repr(self.piths_meta)},',
+            f'    warning_type={repr(self.warning_type)},',
             f')',
         ))
 
@@ -294,8 +294,8 @@ class HintPepMetadata(HintNonpepMetadata):
         **arguments** (i.e., PEP-compliant type hints that are *not* type
         variables) and/or **type variables** (i.e., :class:`typing.TypeVar`
         instances). Defaults to :data:`True` only if the machine-readable
-        representation of this hint contains one or more ``[`` delimiters.
-    is_pep585_builtin_subscripted : bool, optional
+        representation of this hint contains one or more "[" delimiters.
+    is_pep585_builtin_subbed : bool, optional
         :data:`True` only if this hint is a :pep:`585`-compliant builtin. If
         :data:`True`, then :attr:`is_type_typing` *must* be :data:`False`.
         Defaults to the negation of :attr:`is_pep585_generic` if
@@ -308,14 +308,14 @@ class HintPepMetadata(HintNonpepMetadata):
     is_typevars : bool, optional
         :data:`True` only if this hint is subscripted by one or more **type
         variables** (i.e., :class:`typing.TypeVar` instances). Defaults to
-        :data:`False`.
+        ``bool(typevars)``.
     is_type_typing : bool, optional
         :data:`True` only if this hint's class is defined by the :mod:`typing`
-        module. If ``True``, then :attr:`is_pep585_builtin_subscripted` and
+        module. If ``True``, then :attr:`is_pep585_builtin_subbed` and
         :attr:`is_pep585_generic` *must* both be ``False``. Defaults to
         either:
 
-        * If either :attr:`is_pep585_builtin_subscripted` *or*
+        * If either :attr:`is_pep585_builtin_subbed` *or*
           :attr:`is_pep585_generic` are :data:`True`, :data:`False`.
         * Else, :data:`True`.
     is_typing : bool, optional
@@ -338,6 +338,11 @@ class HintPepMetadata(HintNonpepMetadata):
         Concrete :class:`beartype.door.TypeHint` subclass responsible for
         handling this hint if any *or* :data:`None` otherwise (e.g., if the
         :mod:`beartype.door` submodule has yet to support this hint).
+    typevars : Tuple[TypeVar, ...], optional
+        Tuple of the zero or more :pep:`484`-compliant **type variables** (i.e.,
+        :class:`typing.TypeVar` objects) parametrizing this hint. Defaults to
+        the empty tuple, implying this hint to be parametrized by *no* type
+        variables.
 
     All remaining keyword arguments are passed as is to the superclass
     :meth:`HintNonpepMetadata.__init__` method.
@@ -353,42 +358,45 @@ class HintPepMetadata(HintNonpepMetadata):
 
         # Optional keyword-only parameters.
         is_args: Optional[bool] = None,
-        is_pep585_builtin_subscripted: Optional[bool] = None,
+        is_pep585_builtin_subbed: Optional[bool] = None,
         is_pep585_generic: Optional[bool] = None,
-        is_typevars: bool = False,
+        is_typevars: bool = None,  # type: ignore[assignment]
         is_type_typing: Optional[bool] = None,
         is_typing: Optional[bool] = None,
         isinstanceable_type: Optional[type] = None,
         generic_type: Optional[type] = None,
         typehint_cls: Optional[Type['beartype.door.TypeHint']] = None,
+        typevars: Tuple[TypeVar, ...] = (),
         **kwargs
     ) -> None:
 
         # Defer test-specific imports.
         from beartype._data.hint.pep.sign.datapepsigncls import HintSign
+        from beartype._util.hint.utilhintget import get_hint_repr
         from beartype.door import TypeHint
 
         # Validate passed non-variadic parameters.
-        assert isinstance(is_typevars, bool), (
-            f'{repr(is_typevars)} not bool.')
         assert isinstance(pep_sign, HintSign), f'{repr(pep_sign)} not sign.'
         assert isinstance(isinstanceable_type, _NoneTypeOrType), (
             f'{repr(isinstanceable_type)} neither class nor "None".')
+        assert isinstance(typevars, tuple), f'{repr(typevars)} not tuple.'
+        assert all(isinstance(typevar, TypeVar) for typevar in typevars), (
+            f'{repr(typevars)} not tuple of type variables.')
 
         # Initialize our superclass with all remaining variadic parameters.
         super().__init__(**kwargs)
 
         # Machine-readable representation of this hint.
-        hint_repr = repr(self.hint)
+        hint_repr = get_hint_repr(self.hint)
 
         # Conditionally default all unpassed parameters.
-        if is_args is None:
             # Default this parameter to true only if the machine-readable
             # representation of this hint contains "[": e.g., "List[str]".
+        if is_args is None:
             is_args = '[' in hint_repr
-        if is_pep585_builtin_subscripted is None:
-            # Default this parameter to true only if...
-            is_pep585_builtin_subscripted = (
+        # Default this parameter to true only if...
+        if is_pep585_builtin_subbed is None:
+            is_pep585_builtin_subbed = (
                 # This hint originates from an origin type *AND*...
                 isinstanceable_type is not None and
                 # The machine-readable representation of this hint is prefixed
@@ -397,38 +405,44 @@ class HintPepMetadata(HintNonpepMetadata):
                 # 585-compliant builtin.
                 hint_repr.startswith(isinstanceable_type.__name__)
             )
-            # print(f'is_pep585_builtin_subscripted: {is_pep585_builtin_subscripted}')
+            # print(f'is_pep585_builtin_subbed: {is_pep585_builtin_subbed}')
             # print(f'hint_repr: {hint_repr}')
             # print(f'isinstanceable_type.__name__: {isinstanceable_type.__name__}')
+        # Default this parameter to false, because we can't think of anything
+        # better.
         if is_pep585_generic is None:
-            # Default this parameter to false, because we can't think of
-            # anything better.
             is_pep585_generic = False
+        # Default this parameter to the negation of all PEP 585-compliant
+        # boolean parameters. By definition, PEP 585-compliant type hints are
+        # *NOT* defined by the "typing" module and vice versa.
         if is_type_typing is None:
-            # Default this parameter to the negation of all PEP 585-compliant
-            # boolean parameters. By definition, PEP 585-compliant type hints
-            # are *NOT* defined by the "typing" module and vice versa.
             is_type_typing = not (
-                is_pep585_builtin_subscripted or is_pep585_generic)
+                is_pep585_builtin_subbed or is_pep585_generic)
+        # Default this parameter to true only if this hint is expected to be
+        # parametrized by one or more type variables.
+        if is_typevars is None:
+            is_typevars = bool(typevars)
+        # Default this parameter to true only if this hint's class is
+        # defined by the "typing" module.
         if is_typing is None:
-            # Default this parameter to true only if this hint's class is
-            # defined by the "typing" module.
             is_typing = is_type_typing
+        # Default this parameter to this hint's type origin only if this
+        # hint is subscripted.
         if generic_type is None:
-            # Default this parameter to this hint's type origin only if this
-            # hint is subscripted.
             generic_type = isinstanceable_type if is_args else None
 
         # Defer validating parameters defaulting to "None" until *AFTER*
         # initializing these parameters above.
         assert isinstance(is_args, bool), (
             f'{repr(is_args)} not bool.')
-        assert isinstance(is_pep585_builtin_subscripted, bool), (
-            f'{repr(is_pep585_builtin_subscripted)} not bool.')
+        assert isinstance(is_pep585_builtin_subbed, bool), (
+            f'{repr(is_pep585_builtin_subbed)} not bool.')
         assert isinstance(is_pep585_generic, bool), (
             f'{repr(is_pep585_generic)} not bool.')
         assert isinstance(is_type_typing, bool), (
             f'{repr(is_type_typing)} not bool.')
+        assert isinstance(is_typevars, bool), (
+            f'{repr(is_typevars)} not bool.')
         assert isinstance(is_typing, bool), (
             f'{repr(is_typing)} not bool.')
         assert isinstance(generic_type, _NoneTypeOrType), (
@@ -445,21 +459,21 @@ class HintPepMetadata(HintNonpepMetadata):
             f'"beartype.door.TypeHint" subclass nor "None".'
         )
 
-        # Validate that the "is_pep585_builtin_subscripted" and "is_type_typing" parameters
-        # are *NOT* both true. Note, however, that both can be false (e.g., for
-        # PEP 484-compliant user-defined generics).
+        # Validate that the "is_pep585_builtin_subbed" and "is_type_typing"
+        # parameters are *NOT* both true. Note, however, that both can be false
+        # (e.g., for PEP 484-compliant user-defined generics).
         assert not (
-            (is_pep585_builtin_subscripted or is_pep585_generic) and is_type_typing), (
+            (is_pep585_builtin_subbed or is_pep585_generic) and is_type_typing), (
             f'Mutually incompatible boolean parameters '
             f'is_type_typing={repr(is_type_typing)} and either '
-            f'is_pep585_builtin_subscripted={repr(is_pep585_builtin_subscripted)} or '
+            f'is_pep585_builtin_subbed={repr(is_pep585_builtin_subbed)} or '
             f'is_pep585_generic={repr(is_pep585_generic)} enabled.'
         )
 
         # Classify all passed parameters.
         self.generic_type = generic_type
         self.is_args = is_args
-        self.is_pep585_builtin_subscripted = is_pep585_builtin_subscripted
+        self.is_pep585_builtin_subbed = is_pep585_builtin_subbed
         self.is_pep585_generic = is_pep585_generic
         self.is_typevars = is_typevars
         self.is_type_typing = is_type_typing
@@ -467,8 +481,9 @@ class HintPepMetadata(HintNonpepMetadata):
         self.isinstanceable_type = isinstanceable_type
         self.pep_sign = pep_sign
         self.typehint_cls = typehint_cls
+        self.typevars = typevars
 
-    # ..................{ STRINGIFIERS                       }..................
+    # ..................{ DUNDERS                            }..................
     def __repr__(self) -> str:
         return '\n'.join((
             f'{self.__class__.__name__}(',
@@ -481,13 +496,15 @@ class HintPepMetadata(HintNonpepMetadata):
             f'    is_args={repr(self.is_args)},',
             f'    is_ignorable={repr(self.is_ignorable)},',
             f'    is_needs_cls_stack={repr(self.is_needs_cls_stack)},',
-            f'    is_pep585_builtin_subscripted={repr(self.is_pep585_builtin_subscripted)},',
+            f'    is_pep585_builtin_subbed={repr(self.is_pep585_builtin_subbed)},',
             f'    is_pep585_generic={repr(self.is_pep585_generic)},',
             f'    is_supported={repr(self.is_supported)},',
             f'    is_typevars={repr(self.is_typevars)},',
             f'    is_type_typing={repr(self.is_type_typing)},',
             f'    is_typing={repr(self.is_typing)},',
             f'    piths_meta={repr(self.piths_meta)},',
+            f'    typevars={repr(self.typevars)},',
+            f'    warning_type={repr(self.warning_type)},',
             f')',
         ))
 
